@@ -167,6 +167,8 @@ class dirstate(object):
         self._dirty = False
 
     def copy(self, source, dest):
+        if source == dest:
+            return
         self._dirty = True
         self._copymap[dest] = source
 
@@ -407,6 +409,8 @@ class dirstate(object):
         return False
 
     def _dirignore(self, f):
+        if f == '.':
+            return False
         if self._ignore(f):
             return True
         for c in strutil.findall(f, '/'):
