@@ -789,7 +789,7 @@ def selectfile(afile_orig, bfile_orig, hunk, strip, reverse):
         pathlen = len(path)
         i = 0
         if count == 0:
-            return path.rstrip()
+            return '', path.rstrip()
         while count > 0:
             i = path.find('/', i)
             if i == -1:
@@ -1039,9 +1039,12 @@ def applydiff(ui, fp, changed, strip=1, sourcefile=None, reverse=False,
                 continue
         elif state == 'git':
             gitpatches = values
+            cwd = os.getcwd()
             for gp in gitpatches:
                 if gp.op in ('COPY', 'RENAME'):
-                    copyfile(gp.oldpath, gp.path)
+                    src, dst = [util.canonpath(cwd, cwd, x)
+                                for x in [gp.oldpath, gp.path]]
+                    copyfile(src, dst)
                 changed[gp.path] = (gp.op, gp)
         else:
             raise util.Abort(_('unsupported parser state: %s') % state)
@@ -1055,9 +1058,9 @@ def applydiff(ui, fp, changed, strip=1, sourcefile=None, reverse=False,
     return err
 
 def diffopts(ui, opts={}, untrusted=False):
-    def get(key, name=None):
+    def get(key, name=None, getter=ui.configbool):
         return (opts.get(key) or
-                ui.configbool('diff', name or key, None, untrusted=untrusted))
+                getter('diff', name or key, None, untrusted=untrusted))
     return mdiff.diffopts(
         text=opts.get('text'),
         git=get('git'),
@@ -1066,7 +1069,7 @@ def diffopts(ui, opts={}, untrusted=False):
         ignorews=get('ignore_all_space', 'ignorews'),
         ignorewsamount=get('ignore_space_change', 'ignorewsamount'),
         ignoreblanklines=get('ignore_blank_lines', 'ignoreblanklines'),
-        context=get('unified'))
+        context=get('unified', getter=ui.config))
 
 def updatedir(ui, repo, patches):
     '''Update dirstate after patch application according to metadata'''
@@ -1106,7 +1109,7 @@ def updatedir(ui, repo, patches):
             if ctype == 'ADD' and not os.path.exists(dst):
                 repo.wwrite(gp.path, '', flags)
             else:
-                util.set_flags(dst, flags)
+                util.set_flags(dst, 'l' in flags, 'x' in flags)
     cmdutil.addremove(repo, cfiles)
     files = patches.keys()
     files.extend([r for r in removes if r not in files])
